require_gem 'usage'
usage = Usage.new "[-d %max_depth] [-a] [-c] name_regexp [content_regexp]", <<EOT
-d,--depth     the maximum depth to recurse to (defaults to no limit)
-a,--showall   with content_regexp, show every match per file
               (defaults to only show the first-match per file)
-c,--usecase   with content_regexp, use case-sensitive matching
               (defaults to case-insensitive)
EOT

class Dir
  def self.crawl( path, max_depth=nil, depth=0, &block )
    return if max_depth && depth > max_depth
    begin
      if File.directory?( path )
        files = Dir.entries( path ).select{ |f| f[0,1]!='.' }
        unless files.empty?
          files.collect!{ |file_path|
            Dir.crawl( path+'/'+file_path, max_depth, depth+1, &block )
          }.flatten!
        end
        return files
      elsif File.file?( path )
        yield( path, depth )
      end
    rescue SystemCallError => the_error
      warn "ERROR: #{the_error}"
    end
  end

end

start_time = Time.new
name_match = Regexp.new( usage.name_regexp, true )
content_match = usage.content_regexp && Regexp.new( ".{0,20}#{usage.content_regexp}.{0,20}", !usage.usecase )

file_count = 0
matching_count = 0
Dir.crawl( '.', usage.max_depth ){ |file_path, depth|
  if File.split( file_path )[ 1 ] =~ name_match
    if content_match
			if usage.showall
				shown_file = false
				IO.readlines( file_path ).each_with_index{ |line_text,line_number|
					if match = line_text[content_match]
						unless shown_file
							puts file_path
							matching_count += 1
							shown_file = true
						end
						puts ( "%5d:  " % line_number ) + match
					end
				}
				puts " " if shown_file
			elsif IO.read( file_path ) =~ content_match
        puts file_path,"  #{$~}"," "
        matching_count += 1
      end
    else
      puts file_path
      matching_count += 1
    end
  end
  file_count += 1
}
elapsed = Time.new - start_time

puts "Found #{matching_count} file#{matching_count==1?'':'s'} (out of #{file_count}) in #{elapsed} seconds"